@@ -19,9 +19,9 @@
 
 const assert = require('assert');
 
-const error = require('../../error');
 const By = require('../../lib/by').By;
 const CommandName = require('../../lib/command').Name;
+const error = require('../../lib/error');
 const promise = require('../../lib/promise');
 const until = require('../../lib/until');
 const webdriver = require('../../lib/webdriver'),
@@ -70,39 +70,54 @@ describe('until', function() {
           .then(fail, (e2) => assert.strictEqual(e2, e));
     });
 
+    const ELEMENT_ID = 'some-element-id';
+    const ELEMENT_INDEX = 1234;
+
+    function onSwitchFrame(expectedId) {
+      if (typeof expectedId === 'string') {
+        expectedId = WebElement.buildId(expectedId);
+      } else {
+        assert.equal(typeof expectedId, 'number', 'must be string or number');
+      }
+      return cmd => {
+        assert.deepEqual(
+            cmd.getParameter('id'), expectedId, 'frame ID not specified');
+        return true;
+      };
+    }
+
     it('byIndex', function() {
-      executor.on(CommandName.SWITCH_TO_FRAME, () => true);
-      return driver.wait(until.ableToSwitchToFrame(0), 100);
+      executor.on(CommandName.SWITCH_TO_FRAME, onSwitchFrame(ELEMENT_INDEX));
+      return driver.wait(until.ableToSwitchToFrame(ELEMENT_INDEX), 100);
     });
 
     it('byWebElement', function() {
-      executor.on(CommandName.SWITCH_TO_FRAME, () => true);
-      var el = new webdriver.WebElement(driver, {ELEMENT: 1234});
+      executor.on(CommandName.SWITCH_TO_FRAME, onSwitchFrame(ELEMENT_ID));
+
+      var el = new webdriver.WebElement(driver, ELEMENT_ID);
       return driver.wait(until.ableToSwitchToFrame(el), 100);
     });
 
     it('byWebElementPromise', function() {
-      executor.on(CommandName.SWITCH_TO_FRAME, () => true);
+      executor.on(CommandName.SWITCH_TO_FRAME, onSwitchFrame(ELEMENT_ID));
       var el = new webdriver.WebElementPromise(driver,
-          promise.fulfilled(new webdriver.WebElement(driver, {ELEMENT: 1234})));
+          Promise.resolve(new webdriver.WebElement(driver, ELEMENT_ID)));
       return driver.wait(until.ableToSwitchToFrame(el), 100);
     });
 
     it('byLocator', function() {
-      executor.on(CommandName.FIND_ELEMENTS, () => [WebElement.buildId(1234)]);
-      executor.on(CommandName.SWITCH_TO_FRAME, () => true);
+      executor.on(CommandName.FIND_ELEMENTS, () => [WebElement.buildId(ELEMENT_ID)]);
+      executor.on(CommandName.SWITCH_TO_FRAME, onSwitchFrame(ELEMENT_ID));
       return driver.wait(until.ableToSwitchToFrame(By.id('foo')), 100);
     });
 
     it('byLocator_elementNotInitiallyFound', function() {
-      var foundResponses = [[], [], [WebElement.buildId(1234)]];
+      let foundResponses = [[], [], [WebElement.buildId(ELEMENT_ID)]];
       executor.on(CommandName.FIND_ELEMENTS, () => foundResponses.shift());
-      executor.on(CommandName.SWITCH_TO_FRAME, () => true);
+      executor.on(CommandName.SWITCH_TO_FRAME, onSwitchFrame(ELEMENT_ID));
 
       return driver.wait(until.ableToSwitchToFrame(By.id('foo')), 2000)
-          .then(function() {
-            assert.equal(foundResponses.length, 0);
-          });
+          .then(() => assert.deepEqual(foundResponses, []));
     });
 
     it('timesOutIfNeverAbletoSwitchFrames', function() {
@@ -145,6 +160,40 @@ describe('until', function() {
         return alert.dismiss();
       });
     });
+
+    // TODO: Remove once GeckoDriver doesn't throw this unwanted error.
+    // See https://github.com/SeleniumHQ/selenium/pull/2137
+    describe('workaround for GeckoDriver', function() {
+      it('doesNotFailWhenCannotConvertNullToObject', function() {
+        var count = 0;
+        executor.on(CommandName.GET_ALERT_TEXT, function() {
+          if (count++ < 3) {
+            throw new error.WebDriverError(`can't convert null to object`);
+          } else {
+            return true;
+          }
+        }).on(CommandName.DISMISS_ALERT, () => true);
+
+        return driver.wait(until.alertIsPresent(), 1000).then(function(alert) {
+          assert.equal(count, 4);
+          return alert.dismiss();
+        });
+      });
+
+      it('keepsRaisingRegularWebdriverError', function() {
+        var webDriverError = new error.WebDriverError;
+
+        executor.on(CommandName.GET_ALERT_TEXT, function() {
+          throw webDriverError;
+        });
+
+        return driver.wait(until.alertIsPresent(), 1000).then(function() {
+          throw new Error('driver did not fail against WebDriverError');
+        }, function(error) {
+          assert.equal(error, webDriverError);
+        });
+      })
+    });
   });
 
   it('testUntilTitleIs', function() {
@@ -175,6 +224,35 @@ describe('until', function() {
         });
   });
 
+  it('testUntilUrlIs', function() {
+    var urls = ['http://www.foo.com', 'https://boo.com', 'http://docs.yes.com'];
+    executor.on(CommandName.GET_CURRENT_URL, () => urls.shift());
+
+    return driver.wait(until.urlIs('https://boo.com'), 3000).then(function() {
+      assert.deepStrictEqual(urls, ['http://docs.yes.com']);
+    });
+  });
+
+  it('testUntilUrlContains', function() {
+    var urls =
+        ['http://foo.com', 'https://groups.froogle.com', 'http://google.com'];
+    executor.on(CommandName.GET_CURRENT_URL, () => urls.shift());
+
+    return driver.wait(until.urlContains('oogle.com'), 3000).then(function() {
+      assert.deepStrictEqual(urls, ['http://google.com']);
+    });
+  });
+
+  it('testUntilUrlMatches', function() {
+    var urls = ['foo', 'froogle', 'aaaabc', 'aabbbc', 'google'];
+    executor.on(CommandName.GET_CURRENT_URL, () => urls.shift());
+
+    return driver.wait(until.urlMatches(/^a{2,3}b+c$/), 3000)
+        .then(function() {
+          assert.deepStrictEqual(urls, ['google']);
+        });
+  });
+
   it('testUntilElementLocated', function() {
     var responses = [
         [],
@@ -384,7 +462,7 @@ describe('until', function() {
   describe('WebElementCondition', function() {
     it('fails if wait completes with a non-WebElement value', function() {
       let result = driver.wait(
-          new until.WebElementCondition('testing', () => 123), 1000);
+          new webdriver.WebElementCondition('testing', () => 123), 1000);
 
       return result.then(
           () => assert.fail('expected to fail'),
